package com.qmth.teachcloud.mark.service.impl;

import com.qmth.teachcloud.common.entity.MarkPaper;
import com.qmth.teachcloud.common.enums.mark.MarkPaperStatus;
import com.qmth.teachcloud.common.service.MarkPaperService;
import com.qmth.teachcloud.mark.entity.MarkGroup;
import com.qmth.teachcloud.mark.entity.MarkUserGroup;
import com.qmth.teachcloud.mark.service.MarkGroupService;
import com.qmth.teachcloud.mark.service.MarkService;
import com.qmth.teachcloud.mark.service.MarkUserGroupService;
import com.qmth.teachcloud.mark.utils.TaskLock;
import com.qmth.teachcloud.mark.utils.TaskLockUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class MarkServiceImpl implements MarkService {

    @Resource
    private MarkPaperService markPaperService;
    @Resource
    private MarkGroupService markGroupService;
    @Resource
    private MarkUserGroupService markUserGroupService;

    /**
     * 某个评卷分组已申请的评卷任务数量
     *
     * @return
     */
    @Override
    public int applyCurrentCount(MarkGroup markGroup) {
        // todo 正在评卷数量查询 (评卷员和评卷分组分开2个方法)
        TaskLock taskLock = getTaskLock(markGroup);
        int count = 0;
        if (taskLock != null) {
            count = taskLock.count();
        }
        return count;
    }

    private TaskLock getTaskLock(MarkGroup markGroup) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markGroup.getExamId(), markGroup.getPaperNumber());
        if (markPaper.getStatus() == MarkPaperStatus.FORMAL) {
            return TaskLockUtil.getFormalTask(getGroupKey(markGroup));
        }
        return null;
    }

    /**
     * 释放某个评卷员的所有锁定任务
     *
     * @param markUserGroup
     */
    @Override
    public void releaseByMarkUserGroup(MarkUserGroup markUserGroup) {
        TaskLock taskLock = TaskLockUtil.getFormalTask(getGroupKey(markUserGroup));
        taskLock.clear(markUserGroup.getId());
        TaskLock lock = TaskLockUtil.getTrialTask(getGroupKey(markUserGroup));
        lock.clear(markUserGroup.getId());
    }

    @Override
    public int applyCurrentCount(MarkGroup markGroup, Long markUserGroupId) {
        int count = 0;
        TaskLock taskLock = getTaskLock(markGroup);
        if (taskLock != null) {
            count = taskLock.count(markUserGroupId);
        }
        return count;
    }

    @Override
    public void resetMarker(MarkUserGroup markUserGroup) {
        MarkPaper markPaper = markPaperService.getByExamIdAndPaperNumber(markUserGroup.getExamId(), markUserGroup.getPaperNumber());
        if (markPaper == null || markUserGroupService.getById(markUserGroup.getId()) == null) {
            return;
        }
        /*if (markPaper.getStatus() == MarkPaperStatus.FORMAL) {
            // 遍历相关评卷任务的模式
            List<MarkLibrary> list = libraryDao.findByMarkerIdAndStatusNotIn(marker.getId(), LibraryStatus.ARBITRATED,
                    LibraryStatus.WAIT_ARBITRATE, LibraryStatus.PROBLEM);
            for (MarkLibrary library : list) {
                trackDao.deleteByLibraryId(library.getId());
                specialTagDao.deleteByLibraryId(library.getId());
                libraryDao.resetById(library.getId(), null, null, null, null, LibraryStatus.WAITING,
                        library.getStatus());
                lockService.waitlock(LockType.STUDENT, library.getStudentId());
                updateStudentGroupStatus(library.getStudentId(), library.getExamId(), library.getSubjectCode(),
                        library.getGroupNumber(), SubjectiveStatus.UNMARK);
                studentService.updateSubjectiveStatusAndScore(library.getStudentId(), SubjectiveStatus.UNMARK, 0, null);
                inspectedService.cancelByStudent(library.getStudentId());
                lockService.unlock(LockType.STUDENT, library.getStudentId());
            }
            markerDao.resetById(marker.getId());
        }
        updateMarkedCount(group);
        releaseByMarkUserGroup(markUserGroup);
        inspectHistoryService.deleteByExamIdAndSubjectCode(marker.getExamId(), marker.getSubjectCode());*/
    }

    private String getGroupKey(MarkGroup markGroup) {
        return markGroup.getExamId() + "_" + markGroup.getPaperNumber() + "_" + markGroup.getNumber();
    }

    private String getGroupKey(MarkUserGroup markUserGroup) {
        return markUserGroup.getExamId() + "_" + markUserGroup.getPaperNumber() + "_" + markUserGroup.getGroupNumber();
    }
}
